//**************************************************************
// SPACE MINE
//**************************************************************

$TeamDeployableMax[StarscapeMineDeployed] = 30;

datablock ExplosionData(StarscapeMineSubExplosion1) : MortarSubExplosion1
{
   explosionShape = "ss_mortar_explosion.dts";
};             

datablock ExplosionData(StarscapeMineSubExplosion2) : MortarSubExplosion2
{
   explosionShape = "ss_mortar_explosion.dts";
};

datablock ExplosionData(StarscapeMineSubExplosion3) : MortarSubExplosion3
{
   explosionShape = "ss_mortar_explosion.dts";
};

datablock ShockwaveData(StarscapeMineShockwave) : MortarShockwave
{
   colors[0] = "1.0 0.0 0.0 0.50";
   colors[1] = "1.0 0.0 0.0 0.25";
   colors[2] = "1.0 0.0 0.0 0.0";
};

datablock ParticleData(StarscapeMineCrescentParticle) : MortarCrescentParticle
{
   colors[0] = "1.0 0.0 0.0 1.0";
   colors[1] = "1.0 0.0 0.0 0.5";
   colors[2] = "1.0 0.0 0.0 0.0";
};

datablock ParticleEmitterData(StarscapeMineCrescentEmitter) : MortarCrescentEmitter
{
   particles = "StarscapeMineCrescentParticle";
};

datablock ParticleData(StarscapeMineExplosionSmoke) : MortarExplosionSmoke
{
   colors[0] = "1.0 0.0 0.0 0.0";
   colors[1] = "0.7 0.0 0.0 0.5";
   colors[2] = "0.7 0.0 0.0 0.5";
   colors[3] = "0.4 0.4 0.4 0.0";
};

datablock ParticleEmitterData(StarscapeMineExpSmokeEmitter) : MortarExplosionSmokeEmitter
{
   particles = "StarscapeMineExplosionSmoke";
};

datablock ExplosionData(StarscapeMineExplosion) : MortarExplosion
{
   shockwave = StarscapeMineShockwave;
   subExplosion[0] = StarscapeMineSubExplosion1;
   subExplosion[1] = StarscapeMineSubExplosion2;
   subExplosion[2] = StarscapeMineSubExplosion3;
   emitter[0] = StarscapeMineExpSmokeEmitter;
   emitter[1] = StarscapeMineCrescentEmitter;
};

datablock ItemData(StarscapeMineDeployed) : MineDeployed
{
   shapeFile = "mine1.dts";
   explosion = StarscapeMineExplosion;
   underwaterExplosion = UnderwaterMortarExplosion;
   indirectDamage = 1.5;
   damageRadius = 30.0;
   radiusDamageType = $DamageType::Mine;
   kickBackStrength = 4000;
   spacing = 30.0;
   proximity = 20;
   gravityMod = 0.0;
   heatSignature = 1;
};

//**************************************************************
// FUNCTIONS
//**************************************************************

function StarscapeMineDeployed::onThrow(%this, %mine, %thrower)
{
   %mine.armed = false;
   %mine.damaged = 0;
   %mine.detonated = false;
   %mine.depCount = 0;
   %mine.theClient = %thrower.client;
   %mine.mineSource = %thrower.client;
   %mine.getDatablock().schedule(500, "deployMineCheck", %mine, %thrower);
   schedule(120000, %mine, "explodeMine", %mine, true);
}

function StarscapeMineDeployed::deployMineCheck(%data, %mineObj, %player)
{
   // 2-second delay before mine is armed -- let deploy thread play out etc.
   %data.schedule(%data.armTime, "armDeployedMine", %mineObj);

   // check for other deployed mines in the vicinity
   InitContainerRadiusSearch(%mineObj.getWorldBoxCenter(), %data.spacing, $TypeMasks::ItemObjectType);
   while((%found = containerSearchNext()) != 0)
   {
      if(%found == %mineObj)
         continue;

      if(%found.getType() & $TypeMasks::DamagableItemObjectType)
         schedule(100, %mineObj, "explodeMine", %mineObj, true);
      else
         continue;
   }
   serverPlay3D(MineDeploySound, %mineObj.getTransform());
   %mineTeam = %mineObj.sourceObject.team;
   %mineObj.team = %mineteam;

   if($TeamDeployedCount[%mineTeam, StarscapeMineDeployed] > $TeamDeployableMax[StarscapeMineDeployed])
   {
      messageClient(%player.client, '', 'Maximum allowable space mines deployed.');
      schedule(100, %mineObj, "explodeMine", %mineObj, true);
   }
   else
   {
      %data.CheckVicinity(%mineObj);
      %mineObj.setVelocity("0 0 0");
      %mineObj.setTransform(%mineObj.getTransform());
      %mineObj.playThread($PowerThread, "Power");
      AIDeployMine(%mineObj);
      Game.notifyMineDeployed(%mineObj);
   }
}

function StarscapeMineDeployed::armDeployedMine(%data, %mine)
{
   %mine.armed = true;
   %mine.setHeat(1.0);
   $TeamDeployedCount[%mine.sourceObject.team, %data.getName()]++;
}

function StarscapeMineDeployed::CheckVicinity(%data, %mine)
{
   if(%mine.armed)
   {
      if(!%mine.boom)
      {
         %mineLoc = %mine.getWorldBoxCenter();
         %detonateRange = %mine.getDatablock().proximity;
         InitContainerRadiusSearch(%mineLoc, %detonateRange, $TypeMasks::VehicleObjectType);
         while((%tgt = containerSearchNext()) != 0) 
         {
            if(%tgt.team != %mine.team)
            {
               %mine.detonated = true;
               schedule(50, %mine, "explodeMine", %mine, false);
               break;
            }
         }
      }
   }
   if(!%mine.detonated)
      %data.schedule(100, "CheckVicinity", %mine);
}

function StarscapeMineDeployed::onCollision(%data, %obj, %col)
{
   if(!%obj.armed || %obj.boom)
      return;

   if(%col.getType() & $TypeMasks::VehicleObjectType)
   {
      if(%col.team != %obj.team)
         explodeMine(%obj, false);
   }
}

function StarscapeMineDeployed::damageObject(%data, %targetObject, %sourceObject, %position, %amount, %damageType)
{
   if(!%targetObject.armed || %targetObject.boom)
      return;

   %targetObject.damaged += %amount;
   if(%targetObject.damaged >= %data.maxDamage)
      %targetObject.setDamageState(Destroyed);
}

function StarscapeMineDeployed::onDestroyed(%data, %obj, %lastState)
{
   %source = %obj.mineSource.player;
   %obj.boom = true;
   %mineTeam = %obj.team;
   if(%obj.armed)
      $TeamDeployedCount[%mineTeam, %data.getName()]--;

   if(!%obj.noDamage)
      RadiusExplosion(%obj, %obj.getPosition(), %data.damageRadius, %data.indirectDamage, 
                      %data.kickBackStrength, %source, %data.radiusDamageType);

   %obj.schedule(500, "delete");
}
