//******************************************************************************
// -starscape MAP FILE-
// Code by Founder and ZOD
//******************************************************************************

datablock MissionMarkerData(SpawnPointMarker)
{
   catagory = "Misc";
   shapeFile = "light_male.dts";
};

datablock StaticShapeData(SensorBuoy) : SensorMediumPulse
{
   shapeFile = "buoy1.dts";
   isInvincible = true;
   targetNameTag = 'Sensor';
   targetTypeTag = 'Buoy';
   sensorColor = "9 194 255";
};

package starscape
{
   function starscape::preLoad(%firstMission)
   {
      // Need to store the map name for scripts to compare to current mission name
      $Map::MyMap = "starscape";
      // Store the default settings in varibles
      $Map::OldGrav = getGravity();
      $Map::OldMine = Mine.thrownItem;
      $Map::LightMissileMax = LightMaleHumanArmor.max[MissileLauncherAmmo];
      $Map::OldMissileVel = TurretMissile.muzzleVelocity;
      $Map::OldMissileInherit = TurretMissile.velInheritFactor;
      $Map::RepBeamRange = DefaultRepairBeam.beamRange;
      $Map::OldDefaultArmor = $DefaultPlayerArmor;
      // Set our custum values
      $DefaultPlayerArmor = Light;
      Mine.thrownItem = StarscapeMineDeployed;
      LightMaleHumanArmor.max[MissileLauncherAmmo] = 8;
      LightFemaleHumanArmor.max[MissileLauncherAmmo] = 8;
      LightMaleBiodermArmor.max[MissileLauncherAmmo] = 8;
      TurretMissile.muzzleVelocity = 100;
      TurretMissile.velInheritFactor = 0.6;
      DefaultRepairBeam.beamRange = 25;
      // Add our custom objects to the arrays
      addDtsToTsStatics("Logos bioderm logo_bd.dts");
      addDtsToTsStatics("Logos starwolf logo_sw.dts");
   }

   function starscape::InitMap()
   {
      // Activate our physical zones
      %grav1 = NameToID("MissionGroup/Teams/Team1/BaseOne/GravityOne");
      %grav1.activate();
      %grav2 = NameToID("MissionGroup/Teams/Team2/BaseTwo/GravityTwo");
      %grav2.activate();
      // Setup our space gravity
      setGravity(-5);
   }

   function starscape::DeactivateMap()
   {
      // Remove our custom objects from the arrays
      removeDtsFromTsStatics("Logos bioderm logo_bd.dts");
      removeDtsFromTsStatics("Logos starwolf logo_sw.dts");
      // Reset anything we've changed to defaults
      setGravity($Map::OldGrav);
      $DefaultPlayerArmor = $Map::OldDefaultArmor;
      Mine.thrownItem = $Map::OldMine;
      LightMaleHumanArmor.max[MissileLauncherAmmo] = $Map::LightMissileMax;
      LightFemaleHumanArmor.max[MissileLauncherAmmo] = $Map::LightMissileMax;
      LightMaleBiodermArmor.max[MissileLauncherAmmo] = $Map::LightMissileMax;
      TurretMissile.muzzleVelocity = $Map::OldMissileVel;
      TurretMissile.velInheritFactor = $Map::OldMissileInherit;
      DefaultRepairBeam.beamRange = $Map::RepBeamRange;
      // Remove all of our store varibles from memory
      deleteVariables("$Map::*");
   }

   function MissionMarkerData::create(%block)
   {
      switch$(%block)
      {
         case "WayPointMarker":
            %obj = new WayPoint() {
               dataBlock = %block;
            };
            return(%obj);
         case "SpawnSphereMarker":
            %obj = new SpawnSphere() {
               datablock = %block;
            };
            return(%obj);
         case "SpawnPointMarker":
            %obj = new SpawnSphere() {
               dataBlock = %block;
            };
            return(%obj);
         case "AIObjectiveMarker":
            %obj = new AIObjective() {
               datablock = %block;
            };
            return(%obj);
      }
      return(-1);
   }

   function serverCmdSuicide(%client)
   {
      if($Map::MyMap $= "starscape")
      {
         if(isObject(%client.player))
         {
            if(%client.player.isMounted())
            {
               %mount = %client.player.getObjectMount();
               %mount.setDamageLevel(%mount.getDatablock().maxDamage);
            }
         }
      }
      Parent::serverCmdSuicide(%client);
   }

   function CTFGame::pickTeamSpawn(%game, %team) 
   {
      %group = nameToID("MissionCleanup/TeamDrops" @ %team);
      %count = %group.getCount();
      for (%i = 0; %i < %count; %i++)
      {
         %index = mFloor(getRandom(1, (%count - 1)));
         %obj = %group.getObject(%index);
         InitContainerRadiusSearch( %obj.position, 20, $TypeMasks::VehicleObjectType );
         %test = containerSearchNext();
         if(!%test)
         {
            return %obj.getTransform();
            break;
         }
      }
   }

   function DMGame::pickTeamSpawn(%game, %team) 
   {
      CTFGame::pickTeamSpawn(%game, %team);
   }

   function CTFGame::createPlayer(%game, %client, %spawnLoc, %respawn)
   {
      if(isObject(%client.player) && (%client.player.getState() !$= "Dead"))
         return;

      if(%client.team == 0)
         error("Players should not be added to team0!");

      if(%spawnLoc == -1)      
         %spawnLoc = "0 0 9900 1 0 0 0";

      if (%client.race $= "Bioderm")
         %armor = $DefaultPlayerArmor @ "Male" @ %client.race @ Armor;
      else
         %armor = $DefaultPlayerArmor @ %client.sex @ %client.race @ Armor;

      %client.armor = $DefaultPlayerArmor;
      %player = new Player() {
         dataBlock = %armor;
      };
      MissionCleanup.add(%player);

      // setup some info
      %player.setOwnerClient(%client);
      %player.team = %client.team;
      %client.outOfBounds = false;
      %player.setEnergyLevel(%player.getDataBlock().maxEnergy);
      %client.player = %player;
      %player.setInventory(MissileLauncherAmmo, 8);

      // updates client's target info for this player
      %player.setTarget(%client.target);
      setTargetDataBlock(%client.target, %player.getDatablock());
      setTargetSensorData(%client.target, PlayerSensor);
      setTargetSensorGroup(%client.target, %client.team);
      %client.setSensorGroup(%client.team);

      if(%player.lastVehicle)
      {
         %player.lastVehicle.lastPilot = "";
         vehicleAbandonTimeOut(%player.lastVehicle);
         %player.lastVehicle = "";
      }
      if(%client.team == 1)
         %player.lastVehicle = FlyingVehicleData::create(QuadRunnerTwo, %client.team);
      else
         %player.lastVehicle = FlyingVehicleData::create(QuadRunner, %client.team);

      %player.lastVehicle.startFade(1000, 0, false);
      %player.lastVehicle.setTransform(%spawnLoc);
      setTargetSensorGroup(%player.lastVehicle.target, %client.team);
      %player.lastVehicle.lastPilot = %player;
      MissionCleanup.add(%player.lastVehicle);

      %player.getDataBlock().onCollision(%player, %player.lastVehicle, 0);

      //make sure the player has been added to the team rank array...
      %game.populateTeamRankArray(%client);
      %game.playerSpawned(%client.player);
   }

   function DMGame::createPlayer(%game, %client, %spawnLoc, %respawn)
   {
      CTFGame::createPlayer(%game, %client, %spawnLoc, %respawn);
   }

   function DefaultGame::equip(%game, %player)
   {
      Parent::equip(%game, %player);
      for(%i = 0; $InvGrenade[%i] !$= ""; %i++)
         %player.setInventory($NameToInv[$InvGrenade[%i]], 0);

      %player.setInventory(EnergyPack, 1);
      %player.setInventory(Mine, %player.getDataBlock().max[Mine]);
      %player.setInventory(FlareGrenade, %player.getDataBlock().max[FlareGrenade]);
      %player.lastGrenade = "FlareGrenade";
      %player.lastWeapon = (%player.getMountedImage($WeaponSlot) == 0 ) ? "" : %player.getMountedImage($WeaponSlot).getName().item;
      %player.unmountImage($WeaponSlot);
   }

   function DMGame::equip(%game, %player)
   {
      Parent::equip(%game, %player);
      for(%i = 0; $InvGrenade[%i] !$= ""; %i++)
         %player.setInventory($NameToInv[$InvGrenade[%i]], 0);

      %player.setInventory(EnergyPack, 1);
      %player.setInventory(Mine, %player.getDataBlock().max[Mine]);
      %player.setInventory(FlareGrenade, %player.getDataBlock().max[FlareGrenade]);
      %player.lastGrenade = "FlareGrenade";
      %player.lastWeapon = (%player.getMountedImage($WeaponSlot) == 0 ) ? "" : %player.getMountedImage($WeaponSlot).getName().item;
      %player.unmountImage($WeaponSlot);
   }

   function CTFGame::startMatch(%game)
   {
      DefaultGame::startMatch(%game);
      for (%i = 0; %i < ClientGroup.getCount(); %i++) {
         %cl = ClientGroup.getObject(%i);
         if(isObject(%cl.player))
         {
            %cl.setVWeaponsHudActive(1);
            commandToClient(%cl, 'setHudMode', 'Pilot', "Shrike", 0);
         }
      }
   }

   function DefaultGame::clearDeployableMaxes(%game)
   {
      Parent::clearDeployableMaxes(%game);
      for(%i = 0; %i <= %game.numTeams; %i++)
         $TeamDeployedCount[%i, StarscapeMineDeployed] = 0;
   }

   function getAmmoStationLovin(%client)
   {
      if ( !($InvBanList[$CurrentMissionType, Mine]) )
         %client.player.setInventory( Mine, %client.player.getDataBlock().max[Mine] );

      Parent::getAmmoStationLovin(%client);
      %client.player.setInventory(MissileLauncherAmmo, 8);
   }

   function GameConnection::setVWeaponsHudActive(%client, %slot)
   {
      if(isObject(%client.player.getObjectMount()))
      {
         %vehicle = %client.player.getObjectMount().getDatablock().getName();
         if(%vehicle $= "QuadRunner" || %vehicle $= "QuadRunnerTwo")
            commandToClient(%client, 'setHudMode', 'Pilot', "Shrike", 0);
      }
      Parent::setVWeaponsHudActive(%client, %slot);
   }

   function serverCmdSwitchVehicleWeapon(%client, %dir)
   {
      Parent::serverCmdSwitchVehicleWeapon(%client, %dir);
      %vehicle = %client.player.getControlObject();
      if(isObject(%vehicle))
      {
         %name = %vehicle.getDataBlock().getName();
         if(%name $= "QuadRunner" || %name $= "QuadRunnerTwo")
         {
            if(%vehicle.selectedWeapon > 1)
               %vehicle.mountImage(QuadLockingParam, 0);
            else
               %vehicle.unmountImage(0);
         }
      }
   }

   function Player::use(%this, %data)
   {
      if(%this.isPilot())
      {
         %name = %this.getObjectMount().getDatablock().getName();
         if(%name $= "QuadRunner" || %name $= "QuadRunnerTwo")
         {
            if(%data $= "Grenade")
            {
               if(%this.getInventory("FlareGrenade") > 0)
                  return ShapeBase::use( %this, %data );
               else
                   messageClient(%this.client, 'MsgNoFlares', '\c2Kiss your butt goodbye, you\'re out of flares.~wfx/misc/misc.error.wav');

               return( false );
            }
            else if(%data $= "Mine")
            {
               return ShapeBase::use( %this, %data );
            }
            else if(%data $= "Backpack")
            {
               return vehicleBooster(%this.getObjectMount(), %this);
            }
         }
      }
      Parent::use(%this, %data);
   }

   function FlyingVehicle::BoostEmitter(%veh)
   {
      if(isObject(%veh.BoostEmitter))
         %veh.BoostEmitter.delete();

      %veh.BoostEmitter = new ParticleEmissionDummy() {
         position = %veh.getWorldBoxCenter();
         rotation = "1 0 0 0";
         scale = "1 1 1";
         dataBlock = defaultEmissionDummy;
         emitter = HandGrenadeExplosionSmokeEmitter;
         velocity = "1";
      };
      MissionCleanup.add(%veh.BoostEmitter);
      %veh.BoostEmitter.schedule(500, "delete"); 
   }

   function addDtsToTsStatics(%dts)
   {
      %found = 0;
      for(%i = 0; $StaticTSObjects[%i] !$= ""; %i++)
      {
         if($StaticTSObjects[%i] $= %dts)
         {
            %found = 1;
            break;
         }
      }
      if(%found == 0)
      {
         $StaticTSObjects[%i] = %dts;
         $NumStaticTSObjects++;
      }
   }

   function removeDtsFromTsStatics(%dts)
   {
      %count = 0;
      for(%i = 0; $StaticTSObjects[%i] !$= ""; %i++)
      {
         if($StaticTSObjects[%i] !$= %dts)
         {
            %Temp[%count] = $StaticTSObjects[%i];
            %count++;
         }
         else
            $NumStaticTSObjects--;
      }
      for(%j = 0; %j < %count; %j++)
         $StaticTSObjects[%j] = %Temp[%j];
   }

   function VehicleBooster(%veh, %plyr)
   {
      if(isObject(%veh))
      {
         if(%veh.getEnergyLevel() > 200)
         {
            %veh.BoostEmitter();
            %plyr.playAudio(0, MILFireSound);
            %veh.applyImpulse(%veh.position, Vectorscale(Vectornormalize(%veh.getEyeVector()), 25000));
            %veh.setEnergyLevel(0);
         }
         else
            messageClient(%plyr.client, 'MsgLowEnergy', '\c2Energy level is too low.~wfx/misc/misc.error.wav');
      }
   }

   function Beacon::onUse(%data, %obj)
   {
      if($Map::MyMap $= "starscape")
      {
         if(%obj.inStation)
            return;

         %scEyeVec = VectorScale(VectorNormalize(%obj.getEyeVector()), 25.0);
         %eyeEnd = VectorAdd(posFromTransform(%obj.getEyeTransform()), %scEyeVec);
         %rot = rotFromTransform(%obj.getTransform());
         %searchResult = containerRayCast(posFromTransform(%obj.getEyeTransform()), %eyeEnd, $TypeMasks::VehicleObjectType, 0);
         if(%searchResult)
         {
            messageClient( %obj.client, 'MsgMountSuccess', '\c2Auto mounting %1.', %searchResult.getDataBlock().targetNameTag);
            %searchResult.getDataBlock().mountDriver(%searchResult, %obj);
            return;
         }
         else
         {
            messageClient( %obj.client, 'MsgMountFailed', '\c2No vehicle in 25 meter range.');
            Parent::onUse(%data, %obj);
         }
      }
      else
         Parent::onUse(%data, %obj);
   }
};
